extends Node

func _ready() -> void:
	RenderingServer.call_on_render_thread(repro)

func repro() -> void:
	var rd := RenderingServer.get_rendering_device()

	const SHADER := preload("res://compute.glsl")
	var spirv := SHADER.get_spirv()

	var shader := rd.shader_create_from_spirv(spirv)
	var pipeline := rd.compute_pipeline_create(shader, [
		specialization_constant(0, 3),
		specialization_constant(1, 1.0),
	])

	# because of uniform buffer packing, these are single-precision floats padded to 16 bytes each
	var coefficients: PackedFloat32Array = [
		0, 0, 0, 0,
		0, 0, 0, 0,
		1, 0, 0, 0,
	]
	var coefficients_buffer := rd.uniform_buffer_create(coefficients.size() * 4, coefficients.to_byte_array())
	var coefficients_uniform := RDUniform.new()
	coefficients_uniform.uniform_type = RenderingDevice.UNIFORM_TYPE_UNIFORM_BUFFER
	coefficients_uniform.binding = 0
	coefficients_uniform.add_id(coefficients_buffer)

	var values_buffer := rd.storage_buffer_create(32 * 4)
	var values_uniform := RDUniform.new()
	values_uniform.uniform_type = RenderingDevice.UNIFORM_TYPE_STORAGE_BUFFER
	values_uniform.binding = 1
	values_uniform.add_id(values_buffer)

	var uniform_set := rd.uniform_set_create([
		coefficients_uniform,
		values_uniform,
	], shader, 0)

	var compute_list := rd.compute_list_begin()
	rd.compute_list_bind_compute_pipeline(compute_list, pipeline)
	rd.compute_list_bind_uniform_set(compute_list, uniform_set, 0)
	@warning_ignore("integer_division")
	rd.compute_list_dispatch(compute_list, 32 / 8, 1, 1)
	rd.compute_list_end()

	var values := rd.buffer_get_data(values_buffer).to_float32_array()
	print("The first 32 values of x^2 are: ", values)

	rd.free_rid(shader)
	rd.free_rid(coefficients_buffer)
	rd.free_rid(values_buffer)

func specialization_constant(id: int, value: Variant) -> RDPipelineSpecializationConstant:
	var constant := RDPipelineSpecializationConstant.new()
	constant.constant_id = id
	constant.value = value
	return constant
